/************************************************************************/
/* comm.h - Communication Plugin Architecture Header			*/
/************************************************************************/
/*
 *  (c)2003,2009,2010 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _COMM_H_
#define _COMM_H_

#include <errno.h>

#include <cutil/cutil.h>


/************************************************************************/
/* comm library function return codes (public)				*/
/************************************************************************/
#define COMM_RC_SUCCESS				0
#define COMM_RC_WANT_READ			2
#define COMM_RC_WANT_WRITE			3

/* errno derived error codes */
#define COMM_RC_INVALID_ARGUMENT		EINVAL
#define	COMM_RC_MEMORY_ALLOCATION_FAILURE	ENOMEM
#define COMM_RC_NO_DATA				ENODATA
#define COMM_RC_NO_ENTRY			ENOENT
#define COMM_RC_NO_DEVICE			ENODEV
#define	COMM_RC_CALL_AGAIN			EAGAIN

/* TLS/SSL derived error codes */
#define COMM_RC_UNSUPPORTED_PACKET_VERSION	1008
#define COMM_RC_UNEXPECTED_PACKET_LENGTH	1009
#define COMM_RC_INVALID_SESSION			1010
#define COMM_RC_FATAL_ALERT_RECEIVED		1012
#define COMM_RC_UNEXPECTED_PACKET		1015
#define COMM_RC_WARNING_ALERT_RECEIVED		1016
#define COMM_RC_UNEXPECTED_HANDSHAKE_PACKET	1019
#define COMM_RC_DECRYPTION_FAILURE		1024
#define COMM_RC_DECOMPRESSION_FAILURE		1026
#define COMM_RC_COMPRESSION_FAILURE		1027
#define COMM_RC_INSUFFICIENT_CREDENTIALS	1032
#define COMM_RC_UNKNOWN_ERROR			1999


/* internal state flags */
#define COMMFLAG_IO_BLOCK	0x00000001
#define COMMFLAG_SERVER		0x00000002
#define COMMFLAG_CONNECTED	0x00000004
#define COMMFLAG_INACCEPT	0x00000008

/* internal option flags */
#define	COMM_OPTION_TIMEOUT	0x00000001

/* internal I/O flags */
#define	COMM_IO_READ		0x00000001
#define	COMM_IO_WRITE		0x00000002
#define	COMM_IO_EXCEPTION	0x00000004

/* internal pipe file descriptors */
#define	COMM_READ_FD		0
#define	COMM_WRITE_FD		1


#ifndef WIN32
#define	LIBCOMM_API
#else
#ifdef BUILD_LIBCOMM
#define LIBCOMM_API __declspec(dllexport)
#else
#define LIBCOMM_API __declspec(dllimport)
#endif
#endif


typedef struct _CommVectors	CommVectors;
typedef struct _CommModuleData	CommModuleData;
typedef struct _CommModule	CommModule;
typedef struct _CommChannel	CommChannel;
typedef struct _CommGroup	CommGroup;
typedef struct _CommGroupItem	CommGroupItem;

/* comm plugins will need to fill the members of this structure */
struct _CommVectors {
	int (*bind)(CommChannel *cc, SList *interfacelist, int port);
	int (*accept)(CommChannel *cc, CommChannel *ncc, int commflags);
	int (*accept_continue)(CommChannel *cc);
	int (*connect)(CommChannel *cc, char *host, int port);
	int (*read)(CommChannel *cc, char *buffer, int *bufferlen);
	int (*write)(CommChannel *cc, char *buffer, int *bufferlen);
	int (*close)(CommChannel *cc, int commflags);
	int (*get_fd)(CommChannel *cc, int *fdp);
};


typedef int (*CommLoadCallback)(CommModuleData *cmd);
typedef int (*CommUnloadCallback)(CommModuleData *cmd);
typedef int (*CommIOCallback)(CommChannel *cc, int iotype, void *ccarg);
typedef int (*CommTimeoutCallback)(CommGroup *cg, void *arg);


/* Passed to each module in comm_init(), all members, except attribute are
 * to be filled by the module
*/
struct _CommModuleData {
	char			*name;		/* the module's name */
	void			*data;		/* module specific data */
	CommVectors		*vector;	/* module vectors */
	Attribute		*attribute;	/* module attributes */
};


/* One struct for each loaded module, maintained by the plug manager */
struct _CommModule {
	char			*path;		/* the module's path */
	void			*module;	/* dynamic loader module pointer */
	CommLoadCallback	load;		/* module load callback */
	CommUnloadCallback	unload;		/* module unload callback */
	CommModuleData		*moddata;	/* module data structure */
	int			refcount;	/* module use reference counter */
};


struct _CommChannel {
	CommModule	*cmodule;	/* Comm module */
	void		*data;		/* plugin channel specific data */
	void		*sdata;		/* channel select-specific data */
	int		commflags;	/* channel comm-specific flags */
	int		flags;		/* channel specific flags */
	int		ioneeds;	/* channel i/o needs */
	void		*cgdata;	/* channel comm_group data */
};


struct _CommGroup {
	/* base members */
	CommIOCallback		callback;	/* group I/O handler callback */
	SList			*cclist;	/* slist of commchannels */
	unsigned long		flags;		/* group flags */
	int			pipefd[2];	/* group control pipeset */
	int			epfd;
	void			*sdata;		/* private group specific pointer */
	/* option members */
	unsigned long		timeout;
	CommTimeoutCallback	timeoutcallback; /* timeout callback function */
	void			*timeoutarg;	/* timeout callback argument */
};


struct _CommGroupItem {
	CommChannel	*cc;
	void		*ccarg;
	int		ccfd;
	unsigned int	iotype;
	CommGroup	*cg;
};



/* The comm API function prototypes */
LIBCOMM_API int comm_plugin_load (CommModule **cmp, char *path, Attribute *attribs);
LIBCOMM_API int comm_plugin_unload (CommModule **cmp);
LIBCOMM_API int comm_bind (CommModule *cm, CommChannel **ccp,
		SList *interfacelist, int port);
LIBCOMM_API int comm_accept (CommChannel *cc, CommChannel **ccp, int commflags);
LIBCOMM_API int comm_accept_continue (CommChannel *cc);
LIBCOMM_API int comm_connect (CommModule *cm, CommChannel **ccp,
		char *host, int port);
LIBCOMM_API int comm_read (CommChannel *cc, char *buffer, int *bytes);
LIBCOMM_API int comm_write (CommChannel *cc, char *buffer, int *bytes);
LIBCOMM_API int comm_close (CommChannel **ccp, int commflags);
LIBCOMM_API int comm_get_flags (CommChannel *cc, int *commflags);
LIBCOMM_API int comm_set_flags (CommChannel *cc, int commflags);
LIBCOMM_API int comm_get_fd (CommChannel *cc, int *fdp);
LIBCOMM_API int comm_get_ioneeds (CommChannel *cc, int *ioneeds);
LIBCOMM_API int comm_set_ioneeds (CommChannel *cc, int ioneeds);

LIBCOMM_API int comm_group_new (CommGroup **cgp, CommIOCallback cb);
LIBCOMM_API int comm_group_free (CommGroup **cgp);
LIBCOMM_API int comm_group_set_option (CommGroup *cg, int option, unsigned long value);
LIBCOMM_API int comm_group_get_option (CommGroup *cg, int option, unsigned long *value);
LIBCOMM_API int comm_group_add (CommGroup *cg, CommChannel *cc, void *ccarg, unsigned int iotype);
LIBCOMM_API int comm_group_remove (CommGroup *cg, CommChannel *cc);
LIBCOMM_API int comm_group_loop (CommGroup *cg);
LIBCOMM_API int comm_group_exit_loop (CommGroup *cg);
LIBCOMM_API int comm_group_restart_loop (CommGroup *cg);
LIBCOMM_API int comm_group_set_timeout_callback (CommGroup *cg,
		CommTimeoutCallback callback,
		void *arg);
#endif

